#!/bin/bash

generate_ca_csr(){
  cat > $1/ca-csr.json << EOF 
{
  "CN": "cloudops",
  "key": {
    "algo": "rsa",
    "size": 2048
  },
  "names": [
    {
      "C": "US",
      "L": "CA",
      "O": "Cloud Ops - Synopsys",
      "ST": "San Francisco",
      "OU": "SIG"
    }
  ]
}
EOF
}

generate_ca_config(){
  cat > $1/ca-config.json << EOF
{
  "signing": {
    "default": {
      "expiry": "720h"
    },
    "profiles": {
      "server": {
        "expiry": "720h",
        "usages": [
          "signing",
          "key encipherment",
          "server auth"
        ]
      },
      "client": {
        "expiry": "720h",
        "usages": [
          "signing",
          "key encipherment",
          "client auth"
        ]
      }
    }
  }
}
EOF
}

generate_vault_metadata(){
  cat > $1/vault.json << EOF
{
  "CN": "vault",
  "key": {
    "algo": "rsa",
    "size": 2048
  }
}
EOF
}

provision_vault_tls_certificate(){
  mkdir -p environments/${ENVIRONMENT_NAME}/secrets/vault-tls-certificate-new
  if kubectl get secret vault-tls-certificate-new --namespace ${ENVIRONMENT_NAME}; then
    kubectl get secret vault-tls-certificate-new -o json -n ${ENVIRONMENT_NAME} | jq -r '.data."tls.crt"' | base64 --decode > environments/${ENVIRONMENT_NAME}/secrets/vault-tls-certificate-new/tls.crt
    kubectl get secret vault-tls-certificate-new -o json -n ${ENVIRONMENT_NAME} | jq -r '.data."tls.key"' | base64 --decode > environments/${ENVIRONMENT_NAME}/secrets/vault-tls-certificate-new/tls.key
    kubectl get secret vault-tls-certificate-new -o json -n ${ENVIRONMENT_NAME} | jq -r '.data."ca.crt"' | base64 --decode > environments/${ENVIRONMENT_NAME}/secrets/vault-tls-certificate-new/ca.crt
    echo "vault server tls certificate already exists.. skipping generation.."
  else
    echo "Generating certificate singing request.."
    generate_ca_csr "environments/${ENVIRONMENT_NAME}/secrets/vault-tls-certificate-new"
    echo "Generating certificate authority configuration.."
    generate_ca_config "environments/${ENVIRONMENT_NAME}/secrets/vault-tls-certificate-new"
    generate_vault_metadata "environments/${ENVIRONMENT_NAME}/secrets/vault-tls-certificate-new"
    generate_vault_tls_certificate "environments/${ENVIRONMENT_NAME}/secrets/vault-tls-certificate-new"
  fi
}

generate_vault_tls_certificate(){
  path=$1
  echo "Initializing CA certificate.."
  cfssl gencert -initca $path/ca-csr.json | cfssljson -bare $path/ca 
  echo "Generating certificates for vault service.."
  cfssl gencert -ca=$path/ca.pem -ca-key=$path/ca-key.pem -config=$path/ca-config.json -profile=server -hostname="vault,localhost,vault.$NAMESPACE,vault.$NAMESPACE.svc,vault.$NAMESPACE.svc.cluster,vault.$NAMESPACE.svc.cluster.local,127.0.0.1" -loglevel=5 $path/vault.json | cfssljson -bare $path/tls
  mv $path/tls.pem $path/tls.crt
  mv $path/tls-key.pem $path/tls.key
  mv $path/ca.pem $path/ca.crt
  echo "Generated certificates are placed under $path"
}

update_kubernetes_secret(){
  secret_name=$2
  path_to_certs=$1
  echo "Updating $secret_name kubernetes secret.."
  export CA_CRT=$(cat $path_to_certs/ca.crt | base64) TLS_CRT=$(cat $path_to_certs/tls.crt | base64 ) TLS_KEY=$(cat $path_to_certs/tls.key | base64)
  curl -k $K8S_API_SERVER/api/v1/namespaces/$NAMESPACE/secrets/$secret_name --header "Authorization: Bearer $K8S_TOKEN" --cacert /var/run/secrets/kubernetes.io/serviceaccount/ca.crt -X PUT -H "Content-Type: application/json" -H "Accept: application/json" --write-out '%{http_code}' --silent --output /dev/null -d @- <<EOF
  {"metadata": {"name":"$secret_name"},"data":{"ca.crt":"$CA_CRT","tls.crt":"$TLS_CRT","tls.key":"$TLS_KEY"},"type":"kubernetes.io/tls"}
EOF
}

revoke_certs_with_1_year_validity(){
  secret_name=$1
  if kubectl get secret $secret_name -n $NAMESPACE; then
    start=$(kubectl get secret $secret_name -n $NAMESPACE -o json | jq -r '.data."tls.crt"' | base64 --decode | cfssl certinfo -cert - | jq -r '.not_before' | cut -d'T' -f1)
    end=$(kubectl get secret $secret_name -n $NAMESPACE -o json | jq -r '.data."tls.crt"' | base64 --decode | cfssl certinfo -cert - | jq -r '.not_after' | cut -d'T' -f1)
    echo $((($(date -d "$end" "+%s")-$(date -d "$start" "+%s"))/(3600*24)))
    if [[ $((($(date -d "$end" "+%s")-$(date -d "$start" "+%s"))/(3600*24))) -eq 365 ]];then
      echo "$secret_name has certificates valid for 1 year.. Deleting it.."
      kubectl delete secret $secret_name -n $NAMESPACE
    fi
  fi
}