#!/bin/bash

check_vault_service_status(){
  echo "Checking for vault service availability.."
  while true; do
    vault status -format=json > /tmp/status.json
    if [ $? -eq 1 ]; then
      echo "Vault service is unreachable.. sleeping for 5s.."
      sleep 5
    else
      VAULT_INIT_STATUS=$(cat /tmp/status.json | jq -r '.initialized')
      VAULT_SEAL_STATUS=$(cat /tmp/status.json | jq -r '.sealed')
      if [ "$VAULT_INIT_STATUS" == "true" ];then
        if [ "$VAULT_SEAL_STATUS" == "false" ];then
          echo "Vault initialize status : $VAULT_INIT_STATUS"
          echo "Vault seal status : $VAULT_SEAL_STATUS"
          echo "Vault service is available and accepting connections.."
          break
        fi
      fi
    fi
  done
}

login_to_vault(){
  echo "Trying to login into Vault.."
  vault login -no-store $VAULT_TOKEN > /dev/null
}

enable_secrets_engine_in_vault(){
  if [ -z $(vault secrets list -format=json | jq "keys" | jq '.[] | select(. == "secret/")') ]; then
    echo "Enabling secret engine of type 'kv' with version '2' under 'secret' path.."
    vault secrets enable -path=secret -version=2 kv
  fi
}

create_or_update_vault_policies(){
  echo "Creating/Updating auth-server policy.."
  vault policy write auth-server /opt/policies/auth-server.hcl
  echo "Creating auth-client policy.."
  vault policy write auth-client /opt/policies/auth-client.hcl
  echo "Creating tds-code-analysis policy.."
  vault policy write tds-code-analysis /opt/policies/tds-code-analysis.hcl
}

enable_pki_engine_in_vault(){
  if [ -z $(vault secrets list -format=json | jq "keys" | jq '.[] | select(. == "pki/")') ]; then
    echo "Enabling pki engine at 'pki' path.."
    vault secrets enable pki
  fi
}

# Tune secret engine to generate certificates valid for 1 year
tune_pki_secret_engine(){
  echo "Tuning pki secret engine.."
  vault secrets tune -max-lease-ttl=87600h pki
  echo "Generating self-signed root CA certificate"
  vault write -force pki/root/generate/internal common_name=vault ttl=87600h private_key_format="pkcs8"
  echo "Generating URL configurations to include them in certificates.."
  vault write -force pki/config/urls issuing_certificates="${VAULT_ADDR}/v1/pki/ca" crl_distribution_points="${VAULT_ADDR}/v1/pki/crl"
  echo "Generating pki roles.."
  vault write -force pki/roles/client \
    allow_any_name="true" \
    client_flag="true" \
    max_ttl="87600h" \
    server_flag="false"
  vault write -force pki/roles/server \
    allow_any_name="true" \
    allowed_domains="cluster.local" \
    allow_subdomains="true" \
    client_flag="true" \
    max_ttl="87600h" \
    enforce_hostnames="false" \
    server_flag="true"
}

enable_cert_based_authentication_in_vault(){
  if [ -z $(vault auth list -format=json | jq "keys" | jq '.[] | select(. == "cert/")') ]; then
    echo "Enabling cert based authentication for vault.."
    vault auth enable cert
  fi
}

generate_vault_read_only_certificate_for_polaris(){
  echo "Generating certs for auth-client.."
  vault write -force pki/issue/client common_name=auth-client.$NAMESPACE.svc.cluster.local alt_names="auth-client.$NAMESPACE.svc.cluster.local,auth-client,localhost" ip_sans="127.0.0.1" -format=json ttl=720h private_key_format="pkcs8" > /tmp/auth-client-certificates.json
  echo "Extracting certs from auth-client-certificates.json.."
  cat /tmp/auth-client-certificates.json | jq -r '.data.certificate' > /tmp/tls.crt
  cat /tmp/auth-client-certificates.json | jq -r '.data.private_key' > /tmp/tls.key
  cat /tmp/auth-client-certificates.json | jq -r '.data.issuing_ca' > /tmp/ca.crt
  echo "Uploading certificate into vault.."
  vault write -force auth/cert/certs/vault-read-only \
    display_name=vault-read-only \
    policies=auth-client \
    certificate=@/tmp/tls.crt
  create_or_update_kubernetes_secret "vault-read-only"
}

generate_vault_read_write_auth_certificate_for_polaris(){
  echo "Generating certs for auth-server.."
  vault write -force pki/issue/server common_name=auth-server.$NAMESPACE.svc.cluster.local alt_names="auth-server.$NAMESPACE.svc.cluster.local,auth-server,localhost" ip_sans="127.0.0.1" -format=json ttl=720h private_key_format="pkcs8" > /tmp/auth-server-certificates.json
  echo "Extracting certs from auth-server-certificates.json.."
  cat /tmp/auth-server-certificates.json | jq -r '.data.certificate' > /tmp/tls.crt
  cat /tmp/auth-server-certificates.json | jq -r '.data.private_key' > /tmp/tls.key
  cat /tmp/auth-server-certificates.json | jq -r '.data.issuing_ca' > /tmp/ca.crt
  echo "Uploading certificate into vault.."
  vault write -force auth/cert/certs/vault-read-write-auth \
      display_name=vault-read-write \
      policies=auth-server \
      certificate=@/tmp/tls.crt
  create_or_update_kubernetes_secret "vault-read-write-auth"
}

generate_vault_read_write_tds_certificate_for_polaris(){
  echo "Generating certs for tds-code-analysis.."
  vault write -force pki/issue/server common_name=tds-code-analysis.$NAMESPACE.svc.cluster.local alt_names="tds-code-analysis.$NAMESPACE.svc.cluster.local,tds-code-analysis,localhost" ip_sans="127.0.0.1" -format=json ttl=720h private_key_format="pkcs8" > /tmp/tds-code-analysis-certificates.json
  echo "Extracting certs from tds-code-analysis-certificates.json.."
  cat /tmp/tds-code-analysis-certificates.json | jq -r '.data.certificate' > /tmp/tls.crt
  cat /tmp/tds-code-analysis-certificates.json | jq -r '.data.private_key' > /tmp/tls.key
  cat /tmp/tds-code-analysis-certificates.json | jq -r '.data.issuing_ca' > /tmp/ca.crt
  echo "Uploading certificate into vault.."
  vault write -force auth/cert/certs/vault-read-write-tds \
      display_name=vault-read-write-tds \
      policies=tds-code-analysis \
      certificate=@/tmp/tls.crt
  create_or_update_kubernetes_secret "vault-read-write-tds"
}

generate_certificate_for_polaris_webhooks(){
  echo "Generating certificates for polaris webhooks.."
  filename="polaris-webhooks.json"
  vault write -force pki/issue/client common_name=$NAMESPACE.svc.cluster.local alt_names="$NAMESPACE.svc.cluster.local" -format=json ttl=720h private_key_format="pkcs8" > /tmp/$filename
  echo "Extracting certs from $filename.."
  cat /tmp/$filename | jq -r '.data.certificate' > /tmp/tls.crt
  cat /tmp/$filename | jq -r '.data.private_key' > /tmp/tls.key
  cat /tmp/$filename | jq -r '.data.issuing_ca' > /tmp/ca.crt
  create_or_update_kubernetes_secret_tls "polaris-webhook-client-certs"
}

create_or_update_kubernetes_secret(){
  export secretname=$1
  echo "Creating $secretname kubernetes secret.."
  export CA_CRT=$(cat /tmp/ca.crt | base64) TLS_CRT=$(cat /tmp/tls.crt | base64 ) TLS_KEY=$(cat /tmp/tls.key | base64)
  status_code=$(curl -k $K8S_API_SERVER/api/v1/namespaces/$NAMESPACE/secrets/$secretname --header "Authorization: Bearer $K8S_TOKEN" --cacert /var/run/secrets/kubernetes.io/serviceaccount/ca.crt -X GET -H "Content-Type: application/json" --write-out '%{http_code}' --silent --output /dev/null)
  if [[ $status_code -ne 200 ]];then
    curl -k $K8S_API_SERVER/api/v1/namespaces/$NAMESPACE/secrets --header "Authorization: Bearer $K8S_TOKEN" --cacert /var/run/secrets/kubernetes.io/serviceaccount/ca.crt -X POST -H "Content-Type: application/json" -H "Accept: application/json" --write-out '%{http_code}' --silent --output /dev/null -d @- <<EOF
    {"metadata": {"name":"$secretname"},"data":{"ca.crt":"$CA_CRT","tls.crt":"$TLS_CRT","tls.key":"$TLS_KEY"}}
EOF
  else
    curl -k $K8S_API_SERVER/api/v1/namespaces/$NAMESPACE/secrets/$secretname --header "Authorization: Bearer $K8S_TOKEN" --cacert /var/run/secrets/kubernetes.io/serviceaccount/ca.crt -X PUT -H "Content-Type: application/json" -H "Accept: application/json" --write-out '%{http_code}' --silent --output /dev/null -d @- <<EOF
    {"metadata": {"name":"$secretname"},"data":{"ca.crt":"$CA_CRT","tls.crt":"$TLS_CRT","tls.key":"$TLS_KEY"}}
EOF
  fi
}

create_or_update_kubernetes_secret_tls(){
  export secretname=$1
  echo "Creating $secretname kubernetes secret.."
  export CA_CRT=$(cat /tmp/ca.crt | base64) TLS_CRT=$(cat /tmp/tls.crt | base64 ) TLS_KEY=$(cat /tmp/tls.key | base64)
  status_code=$(curl -k $K8S_API_SERVER/api/v1/namespaces/$NAMESPACE/secrets/$secretname --header "Authorization: Bearer $K8S_TOKEN" --cacert /var/run/secrets/kubernetes.io/serviceaccount/ca.crt -X GET -H "Content-Type: application/json" --write-out '%{http_code}' --silent --output /dev/null)
  if [[ $status_code -ne 200 ]];then
    curl -k $K8S_API_SERVER/api/v1/namespaces/$NAMESPACE/secrets --header "Authorization: Bearer $K8S_TOKEN" --cacert /var/run/secrets/kubernetes.io/serviceaccount/ca.crt -X POST -H "Content-Type: application/json" -H "Accept: application/json" --write-out '%{http_code}' --silent --output /dev/null -d @- <<EOF
    {"metadata": {"name":"$secretname"},"data":{"ca.crt":"$CA_CRT","tls.crt":"$TLS_CRT","tls.key":"$TLS_KEY"},"type":"kubernetes.io/tls"}
EOF
  else
    curl -k $K8S_API_SERVER/api/v1/namespaces/$NAMESPACE/secrets/$secretname --header "Authorization: Bearer $K8S_TOKEN" --cacert /var/run/secrets/kubernetes.io/serviceaccount/ca.crt -X PUT -H "Content-Type: application/json" -H "Accept: application/json" --write-out '%{http_code}' --silent --output /dev/null -d @- <<EOF
    {"metadata": {"name":"$secretname"},"data":{"ca.crt":"$CA_CRT","tls.crt":"$TLS_CRT","tls.key":"$TLS_KEY"},"type":"kubernetes.io/tls"}
EOF
  fi
}